(*
////////////////////////////////////////////////////////////////////////////////
// DBFLPROP.PAS : DBFILTERCOMBO PROPERTY EDITORS                              //
// Copyright  1996-1999 Steve Flynn. All Rights Reserved                     //
//----------------------------------------------------------------------------//
// By using this source code you agree to be bound by the terms of the        //
// following Software License Agreement. Any breach of this agreement will be //
// prosecuted to the full extent of the law.                                  //
////////////////////////////////////////////////////////////////////////////////

LICENSE AGREEMENT - MARCH 1999

USE OF THE SOFTWARE

Steven M. Flynn (the "Author") grants you, the end user, a non-exclusive
single-user license to use the supplied software program and all associated
materials including the Source Code (the "Software"). The Software and all
Intellectual Property embodied in the Software always remains the property of
the Author. Your use of the Software indicates your acceptance in full of the
terms and conditions of this agreement.


COPYING THE SOFTWARE

You may freely copy the Software provided that such copies do not infringe upon
any other part of this agreement.


DISTRIBUTION OF THE SOFTWARE

The Software may be re-distributed royalty free providing that it is not
modified in any way and that no charge is made for the Software itself. Only a
nominal fee for media and/or copying services is allowed. All files, including
the License Agreement, that are supplied by the Author with the Software must
also be included in any such distribution.


DERIVED WORKS

You may NOT use the Software to derive any works which are based substantially
in part or in full on the Software or any portion of the Software. Any such
derived works will remain the property of the Author.


MODIFIED VERSIONS OF THE SOFTWARE

All updated, revised, modified or improved versions of the Software are also
subject to the conditions of this license agreement and shall remain the
property of the Author. You are required under the terms of this agreement to
submit to the Author any and all improvements or additions made by you to the 
Software for future inclusion in the general distribution of the Software by
the Author.


TERMINATION OF THIS AGREEMENT

You may terminate this License Agreement at any time by destroying all copies
of the Software in your possession.


LIMITED WARRANTY

The Software is distributed and licensed "AS IS". The Author specifically
disclaims all other warranties, express or implied, including but not limited
to, implied warranties of merchantability and fitness for a particular purpose,
with regard to the Software.


DAMAGES

By using the Software you do so at your own risk. In no event shall the Author
be responsible for any damages whatsoever (including but not limited to,
damages for loss of business profits, business interruption, loss of business
information, or any other pecuniary loss or any other real or consequential 
damages) arising out of the use or inability to use this product.


LIABILITY

In the event of failure of the Software, for any reason, the Author's sole
liability shall be to refund to you the amount paid by you to the Author for
the use of the Software.


COPYRIGHT

The Software and the Intellectual Property embodied in the Software shall
always remain the property of the Author and is protected by Australian
Copyright Law and International Treaty Provisions.


AGREEMENT

Your use of the Software indicates your agreement with the above terms and
conditions. If you do not agree with these terms and conditions then you must
destroy all copies of the software in your possession.


CONSUMER RIGHTS

If any clause in the above Software License Agreement is in violation of your
rights under your local laws then you must cease using the Software
immediately. It is your own responsibility to be aware of your legal rights
under your own local laws.


END OF SOFTWARE LICENSE AGREEMENT

*)

{$A-,B-,D-,F-,G+,I-,L-,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y-,Z-}
{$IFDEF WIN32}
{$H+,J+}
{$M 16384,1048576}
{$IMAGEBASE $00400000}
{$ELSE}
{$K-}
{$M 16384,8192}
{$ENDIF}
{$C DEMANDLOAD}

unit DBFLProp;

interface

uses
	Classes, DsgnIntf;

type
	TComboEditor = class(TDefaultEditor)
	public
		procedure ExecuteVerb(Index: Integer); override;
		function GetVerb(Index: Integer): string; override;
		function GetVerbCount: Integer; override;
		end;


	TDBFCLookupProperty = class(TStringProperty)
	public
		function GetAttributes: TPropertyAttributes; override;
		procedure GetValueList(List: TStrings); virtual; abstract;
		procedure GetValues(Proc: TGetStrProc); override;
		end;


	TDBFCLookupField = class(TDBFCLookupProperty)
	public
		function GetDataSourcePropName: string; virtual;
		function GetLookupSourcePropName: string; virtual;
		procedure GetValueList(List: TStrings); override;
		end;


	TDBFCLookupFilterProperty = class(TDBFCLookupField)
		public
		function GetDataSourcePropName: string; override;
		end;


procedure Register;

implementation

uses
{$IFNDEF WIN32}
	WinTypes,
{$ENDIF}
	DBFltCom, SysUtils, WinProcs, Controls, TypInfo, DB;


{// Property Editor ///////////////////////////////////////////////////////////}
function TDBFCLookupProperty.GetAttributes: TPropertyAttributes;
	begin
	Result := [paValueList, paSortList, paMultiSelect];
	end;


procedure TDBFCLookupProperty.GetValues(Proc: TGetStrProc);
	var
	I: Integer;
	Values: TStringList;
	begin
	Values := TStringList.Create;
	try
		GetValueList(Values);
		for I := 0 to Values.Count - 1 do
			Proc(Values[I]);
	finally
		Values.Free;
		end;
	end;


function TDBFCLookupField.GetDataSourcePropName: string;
	begin
	Result := 'DataSource';
	end;


function TDBFCLookupField.GetLookupSourcePropName: string;
	begin
	Result := 'LookupSource';
	end;


procedure TDBFCLookupField.GetValueList(List: TStrings);
	var
	Instance: TComponent;
	LookupInfo: PPropInfo;
	LookupSource: TDataSource;
	begin
	Instance := TComponent(GetComponent(0));
	LookupInfo := TypInfo.GetPropInfo(Instance.ClassInfo, GetLookupSourcePropName);
	if (LookupInfo <> nil)
		and (LookupInfo^.PropType^.Kind = tkClass) then
		begin
		LookupSource := TObject(GetOrdProp(Instance, LookupInfo)) as TDataSource;
		if (LookupSource <> nil) and (LookupSource.DataSet <> nil) then
			LookupSource.DataSet.GetFieldNames(List);
		end;
	end;


function TDBFCLookupFilterProperty.GetDataSourcePropName: string;
	begin
	Result := 'LookupSource';
	end;


{// TComboEditor //////////////////////////////////////////////////////////}
const
	ComboVerb = 'About DBFilterCombo...';


procedure TComboEditor.ExecuteVerb(Index: Integer);
	var
	ps: array [0..255] of char;
	begin
	MessageBox(0, StrPCopy(ps, TDBFilterCombo.GetCopyright), ComboVerb, MB_OK);
	end;


function TComboEditor.GetVerb(Index: Integer): string;
	begin
	Result := ComboVerb;
	end;


function TComboEditor.GetVerbCount: Integer;
	begin
	Result := 1;
	end;


{// Registration //////////////////////////////////////////////////////////////}
procedure Register;
	begin
	RegisterComponentEditor(TDBFilterCombo, TComboEditor);
	RegisterPropertyEditor(TypeInfo(string), TDBFilterCombo, 'LookupFilterField', TDBFCLookupFilterProperty);
	RegisterPropertyEditor(TypeInfo(string), TDBFilterCombo, 'LookupField', TDBFCLookupField);
	end;

end.

